// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawn.hpp"

namespace Dawn {
  class Scene;

  class Game : public std::enable_shared_from_this<Game> {
    private:
      std::shared_ptr<Scene> currentScene;

    public:
      /**
       * Constructs a new instance of Game, the class majorily responsible for
       * handling all internal dawn functions outside of platform-centric things
       * like windowing and graphics.
       */
      Game();

      /**
       * Initializes the game and all sub managers used by game.
       */
      void init();

      /**
       * Updates the game and all sub managers used by game.
       */
      void update();

      /**
       * Returns the current active scene.
       * 
       * @return The current active scene.
       */
      std::shared_ptr<Scene> getCurrentScene();

      /**
       * Sets the active scene for the game.
       * 
       * @param scene The scene to set as active.
       */
      void setActiveScene(std::shared_ptr<Scene> scene);

      /**
       * Deloads game and all sub managers used by game.
       */
      ~Game();
  };
}