// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma ocne
#include "scene/item/SceneItem.hpp"
#include "game/Game.hpp"

namespace Dawn {
  class Scene : public SceneItem, public std::enable_shared_from_this<Scene> {
    private:
      std::weak_ptr<Game> game;

      /**
       * Invoked by the game whenever this scene has been set as the active 
       * scene.
       */
      void active();

      /**
       * Invoked by the game whenever this scene has been set as the inactive 
       * scene.
       */
      void inactive();

    public:
      /**
       * Creates a new Scene.
       * 
       * @param game The Game this Scene belongs to.
       */
      Scene(std::weak_ptr<Game> game);

      /**
       * Returns true if this scene is the active scene.
       * 
       * @return True if this scene is the active scene.
       */
      bool_t isActiveScene();

      /**
       * Destroys this scene and all of its items.
       */
      virtual ~Scene();

      friend class Game;
  };
}