// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawn.hpp"

/**
 * Returns true if the haystack string includes the needle string.
 * @param haystack The string to search in.
 * @param needle The string to search for.
 * @return True if the needle is found in the haystack.
 */
bool_t stringIncludes(
  const std::string &haystack,
  const std::string &needle
);

/**
 * Converts a string to lowercase.
 * @param str The string to convert.
 * @return The lowercase version of the string.
 */
std::string stringToLowercase(const std::string &str);

/**
 * Splits a string by a delimiter.
 * @param str The string to split.
 * @param delim The delimiter to split by.
 * @return A vector of strings split by the delimiter.
 */
std::vector<std::string> stringSplit(
  const std::string &str,
  const std::string &delim
);