// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnopengl.hpp"
#include "dawn.hpp"
#include <GLFW/glfw3.h>
#include "game/Game.hpp"
#include "display/BackBuffer.hpp"

#ifndef DAWN_GLFW_WIDTH_DEFAULT
  #define DAWN_GLFW_WIDTH_DEFAULT 800
#endif

#ifndef DAWN_GLFW_HEIGHT_DEFAULT
  #define DAWN_GLFW_HEIGHT_DEFAULT 600
#endif

namespace Dawn {
  enum class DawnGLFWInitResult {
    SUCCESS = 0,
    GLFW_INIT_FAILED = 1,
    WINDOW_CREATE_FAILED = 2,
    GLAD_GL_CHECK_FAILED = 3,
    VIEWPORT_SIZE_FAILED = 4
  };

  class DawnGLFWHost {
    private:
      GLFWwindow *window;
      int32_t windowWidth, windowHeight;
      std::shared_ptr<Game> game;
      std::shared_ptr<BackBuffer> backBuffer;

    public:
      /**
       * Creates a new GLFW Host.
       */
      DawnGLFWHost();

      /**
       * Initializes the GLFW Host.
       * 
       * @param argc The number of arguments.
       * @param argv The arguments.
       * @return The result of the initialization.
       */
      DawnGLFWInitResult init(const int32_t argc, const char_t **argv);

      /**
       * Starts the GLFW Host, called after the GLFW Host has been initialized.
       */
      void start();

      /**
       * Destroys the GLFW Host.
       */
      ~DawnGLFWHost();
  };
}