// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "dawnopengl.hpp"
#include "error/assert.hpp"
#include "error/assertgl.hpp"
#include "BackBuffer.hpp"

using namespace Dawn;

BackBuffer::BackBuffer() {

}

float_t BackBuffer::getScale() {
  return this->scale;
}

float_t BackBuffer::getWidth() {
  return this->width;
}

float_t BackBuffer::getHeight() {
  return this->height;
}

void BackBuffer::setSize(
  const float_t width,
  const float_t height
) {
  if(this->width == width && this->height == height) return;
  
  // Fixes a new bug that it seems GLFW has introduced.
  this->width = width == 0 ? 1 : width;
  this->height = height == 0 ? 1 : height;
  
  onResize.emit(this->width, this->height);
}

void BackBuffer::setClearColor(const struct Color color) {
  this->clearColor = color;
}

void BackBuffer::clear(const int32_t clearFlags) {
  glClearColor(clearColor.r, clearColor.g, clearColor.b, 1.0f);
  assertNoGLError();

  GLbitfield mask = 0;
  if((clearFlags & RENDER_TARGET_CLEAR_COLOR) == RENDER_TARGET_CLEAR_COLOR) {
    mask |= GL_COLOR_BUFFER_BIT;
  }

  if((clearFlags & RENDER_TARGET_CLEAR_DEPTH) == RENDER_TARGET_CLEAR_DEPTH) {
    mask |= GL_DEPTH_BUFFER_BIT;
  }

  glClear(mask);
  assertNoGLError();
}

void BackBuffer::bind() {
  #if DAWN_OPENGL_FRAMEBUFFERS
    glBindFramebuffer(GL_FRAMEBUFFER, 0);
    assertNoGLError();
  #endif
  glViewport(0, 0, (GLsizei)this->width, (GLsizei)this->height);
  assertNoGLError();
}