// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "erroropengl.hpp"

void errorCheckOpenGL() {
  std::stringstream buffer;
  GLenum errorCode;
  int32_t errorCount = 0;

  while((errorCode = glGetError()) != GL_NO_ERROR) {
    errorCount++;
    
    switch (errorCode) {
      case GL_INVALID_ENUM:
        buffer << "INVALID_ENUM" << std::endl;
        break;

      case GL_INVALID_VALUE:
        buffer << "INVALID_VALUE" << std::endl;
        break;
      
      case GL_INVALID_OPERATION:
        buffer << "INVALID_OPERATION" << std::endl;
        break;

      #ifdef GL_INVALID_FRAMEBUFFER_OPERATION
        case GL_INVALID_FRAMEBUFFER_OPERATION:
          buffer << "INVALID_FRAMEBUFFER_OPERATION" << std::endl;
          break;
      #endif
      
      case GL_OUT_OF_MEMORY:
        buffer << "OUT_OF_MEMORY" << std::endl;
        break;
      
      case GL_STACK_UNDERFLOW:
        buffer << "STACK_UNDERFLOW" << std::endl;
        break;
      
      case GL_STACK_OVERFLOW:
        buffer << "STACK_OVERFLOW" << std::endl;
        break;

      default:
        buffer << "UNKNOWN" << std::endl;
        break;
    }

    buffer << "(Error Code: " << errorCode << ")" << std::endl;
  }

  if(errorCount > 0) {
    throw std::runtime_error("OpenGL Error(s) Found: " + buffer.str());
  }
}