// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "game/Game.hpp"

namespace Dawn {
  enum class DawnPSPHostResult {
    SUCCESS = 0,
  };

  class DawnPSPHost {
    private:
      std::shared_ptr<Game> game;

    public:
      bool_t shouldGameRun = true;

      /**
       * Creates a new PSP Host.
       */
      DawnPSPHost();

      /**
       * Initializes the PSP Host.
       * 
       * @return The result of the initialization.
       */
      DawnPSPHostResult init();

      /**
       * Starts the PSP Host, called after the PSP Host has been initialized.
       */
      void start();

      /**
       * Destroys the PSP Host.
       */
      ~DawnPSPHost();
  };
}