// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "main.hpp"
#include "host/DawnPSPHost.hpp"

using namespace Dawn;

std::shared_ptr<DawnPSPHost> host;

int callbackOnExit(int arg1, int arg2, void *common) {
  sceKernelExitGame();
  host->shouldGameRun = false;
  host = nullptr;
  return 0;
}

int callbackThread(SceSize args, void *argp) {
  int cbid = sceKernelCreateCallback("Exit Callback", callbackOnExit, NULL);
  sceKernelRegisterExitCallback(cbid);
  sceKernelSleepThreadCB();
  return 0;
}

int main(void)  {
  // Setup PSP Callbacks
  int thid = sceKernelCreateThread(
    "update_thread",
    callbackThread,
    0x11,
    0xFA0,
    0,
    0
  );
  if(thid >= 0) sceKernelStartThread(thid, 0, 0);

  //
  host = std::make_shared<DawnPSPHost>();
  
  auto ret = host->init();
  if(ret != DawnPSPHostResult::SUCCESS) {
    host = nullptr;
    return (int)ret;
  }

  host->start();
  host = nullptr;

  return 0;
}